------------------------------------------------------------------------------
--  Ada95 Interface to Oracle RDBMS                                         --
--  Copyright (C) 2000-2001 Dmitriy Anisimkov.                              --
--  License text and authors contact information could be found in oci.ads  --
------------------------------------------------------------------------------
with
   Ada.Exceptions,
   OCI.Thread;
with Text_IO,System.Address_Image; use Text_IO,System;
package body Oci.Thick is
   use Lib;
   use type Sword;
   use type Ub4;
   use type OCIHandle;
   use Ada.Exceptions;
   
-- Local utilities

   procedure Task_Dedicated_Environment is
   begin
      if not Thread.Synch.Task_Dedicated_Environment then
         raise Too_Late_For_Dedicated_Environment;
      end if;
   end;
   
   procedure Task_Done (T : Task_ID := Current_Task)
   is
   begin
      Thread.Attributes.Reinitialize (T);
   end;  
 
   function Last_Error_Code return Integer
   is
      Rc : Sword;
      Errcodep : aliased Sb4 := 0;
   begin
       Rc := OCIErrorGet(Hndlp => Thread.Error,
               Errcodep => Errcodep'access,
               Bufp => CStr.null_ptr,
               Bufsiz => 0,
               Htype => OCI_HTYPE_ERROR);
       return Integer(Errcodep);
   end;
   
   function Last_Error_Message (Record_No : Integer := 1) return String
   is
      Rc : Sword;
      Errcodep : aliased Sb4 := 0;
      Bufp : aliased C.Char_Array := (0 .. 4095 => C.nul);
   begin
      Rc := OCIErrorGet (Hndlp => Thread.Error,
               Errcodep => Errcodep'access,
               Recordno => Ub4 (Record_No),
               Bufp => CStr.To_Chars_Ptr(Bufp'Unchecked_Access),
               Bufsiz => Bufp'Length-1,
               Htype => OCI_HTYPE_ERROR);
      return C.To_Ada(Bufp);
   end;
 
   function Last_Error_Offset return Natural
   is
   begin
      return Get_Attr (OCIHandle (Thread.Error),
         OCI_HTYPE_ERROR,
         OCI_ATTR_DML_ROW_OFFSET);
   end Last_Error_Offset;

   procedure Check_Error(
         Code : Sword;
         Handle : OCIError;
         Htype : Integer)
         is
      Bufp : aliased C.Char_Array := (0 .. 1023 => C.nul);
      Errcodep : aliased Sb4 := 0;
      Rc : Sword;
   begin
      case Code is
         when OCI_ERROR | OCI_NO_DATA =>
            Rc := OCIErrorGet (Hndlp => Handle,
               Errcodep => Errcodep'access,
               Bufp => CStr.To_Chars_Ptr (Bufp'Unchecked_Access),
               Bufsiz => Bufp'Length - 1,
               Htype => UB4 (Htype));
            if Rc=OCI_SUCCESS then
               Raise_Exception (LIB_ERROR'Identity, C.To_Ada (Bufp));
            else
               Raise_Exception (LIB_ERROR'Identity,
                  "Error code:" & Sword'Image(Code) & Ascii.Lf &
                  "Return error code:" & Sword'Image (Rc) & Ascii.Lf&
                  "Output error code:" & Sb4'Image (Errcodep) & Ascii.Lf&
                  "Message:" & C.To_Ada (Bufp));
            end if;
         when OCI_INVALID_HANDLE => raise INVALID_HANDLE;
         --Later     when OCI_SUCCESS => null;
         when others => null;
      end case;
   end Check_Error;

   procedure Check_Error(
         Code : Sword) is
   begin
      Check_Error(Code, Thread.Error,
         OCI_HTYPE_ERROR);
   end Check_Error;

   function Alloc_Handle(Parent : OCIEnv; Htype : Ub4) return OCIHandle
         is
      Result : aliased OCIHandle := Empty_Handle;
      Rc : Sword := OCIHandleAlloc(Parenth => OCIHandle(Parent),
         Hndlpp => Result'access,
         Htype => Htype);
   begin
      if Rc/=OCI_SUCCESS then
         raise INVALID_HANDLE;
      end if;
      return Result;
   end;

   function Alloc_Descriptor (Parent : OCIEnv; Htype : Ub4) return OCIHandle
   is
      Result : aliased OCIHandle := Empty_Handle;
      Rc : Sword := OCIDescriptorAlloc(Parenth => Parent,
         Descpp => Result'access,
         Htype => Htype);
   begin
      if Rc /= OCI_SUCCESS then
         raise INVALID_HANDLE;
      end if;
      return Result;
   end;

   procedure Free (H : OCIHandle; HType : Ub4) is
      Rc : Sword;
   begin
      if H = Empty_Handle then
         return;
      end if;
--      Put("OCIHandle "&Ub4'Image(HType));   
      RC := OCIHandleFree (H, HType);
--      Put_Line(" freed "&SWord'Image(Rc)); 
      if Rc /= OCI_SUCCESS then
        Put_Line (Standard_Error, "INVALID_HANDLE " & Ub4'Image (HType)); 
  --      raise INVALID_HANDLE;
      end if;
   end Free;
   
   function Get_Attr(Param : OCIHandle; HType : ub4; Attr : Ub4) return String
         is
      Buff : aliased Cstr.Chars_Ptr;
      Rsize : aliased Ub4;
      Rc : Sword := OCIAttrGet (Trgthndlp => Param,
         Trghndltyp => HType,
         Attributep => Buff'access,
         Sizep => Rsize'access,
         Attrtype => Attr,
         Errhp => Thread.Error);
       
   begin
      Check_Error(Rc);
      if cstr."=" (Buff, cstr.Null_Ptr) then
         return "";
      else
         return C.To_Ada (Cstr.Value (Buff, C.Size_T (Rsize)), False);
      end if;  
   end Get_Attr;

   function Get_Attr(Param : OCIHandle; HType : ub4; Attr : Ub4) return Integer
         is
      Result : aliased Integer := 0;
      Rsize : aliased Ub4;
      Rc : Sword := OCIAttrGet (Trgthndlp => Param,
         Trghndltyp => HType,
         Attributep => Result'access,
         Sizep => Rsize'access,
         Attrtype => Attr,
         Errhp => Thread.Error);
   begin
      Check_Error(Rc);
      return Result;
   end Get_Attr;

   procedure Set_Attr(H : OCIHandle; HType : ub4; Value : String; Attr : Ub4)
   is
      rc : SWord := OCIAttrSet (Trgthndlp => H,
         Trghndltyp => HType,
         Attributep => C.To_C (Value),
         Size => Value'Length,
         Attrtype => Attr,
         Errhp => Thread.Error);
   begin
      Check_Error(Rc);
   end Set_Attr;


   procedure Set_Attr (H : OCIHandle; HType : ub4; Value : OCIHandle; Attr : Ub4)
   is
      rc : SWord := OCIAttrSet (Trgthndlp => H,
         Trghndltyp => HType,
         Attributep => Value,
         Size => 0,
         Attrtype => Attr,
         Errhp => Thread.Error);
   begin
      Check_Error(Rc);
   end Set_Attr;



-- Base_Variable
   
   procedure Clear_Value (Var : in out Limited_Variable'Class) is
   begin
      Var.Indicator := Null_Indicator;
   end;

   function Is_Null (Var : Limited_Variable'Class) return Boolean is
      use type Sb2;
   begin
      if Var.Define=OCIDefine(Empty_Handle)
            and Var.Bind=OCIBind(Empty_Handle) then
         raise NOT_ATTACHED;
      end if;
      return Var.Indicator = Null_Indicator;
   end;

-- OCIDate

   function To_String(From : OCIDate; Format : String) return String
         is
      Buff : aliased Text := (0..C."+"(Format'Length,64)=> C.nul);
      Len : aliased Ub4 := Buff'Length-1;
      Rc : Sword := OCIDateToText (
         Err => Thread.Error,
         Date => From,
         Fmt => C.To_C(Format),
         Fmt_Length => Format'Length,
         Lang_Name => 0,
         Lang_Length => 0,
         Buf_Size => Len'access,
         Buf => CStr.To_Chars_Ptr(Buff'Unchecked_Access));
   begin
      Check_Error(Rc);
      return C.To_Ada(Buff,False)(1..Integer(Len));
   end To_String;

   function SysDate return OCIDate is
      Result : aliased OCIDate;
      Rc :  Sword := OCIDateSysDate (Err => Thread.Error,
         Date => Result'access);
   begin
      Check_Error(Rc);
      return Result;
   end SysDate;

   function Handle(ref : Handle_REference'Class) return OCIHandle is
   begin
     return ref.Handle;
   end;

 
end Oci.Thick;